@@ -76,7 +76,6 @@ static gchar *e_book_backend_ldap_build_query (EBookBackendLDAP *bl, const char
 
 static EBookBackendClass *e_book_backend_ldap_parent_class;
 typedef struct _EBookBackendLDAPCursorPrivate EBookBackendLDAPCursorPrivate;
-typedef struct _EBookBackendLDAPBookView EBookBackendLDAPBookView;
 typedef struct LDAPOp LDAPOp;
 
 
@@ -126,15 +125,6 @@ struct _EBookBackendLDAPCursorPrivate {
 	long       num_elements;
 };
 
-struct _EBookBackendLDAPBookView {
-	EDataBookView           *book_view;
-	EBookBackendLDAPPrivate *blpriv;
-	gchar                 *search;
-	int                    limit;
-
-	LDAPOp                *search_op;
-};
-
 typedef void (*LDAPOpHandler)(LDAPOp *op, LDAPMessage *res);
 typedef void (*LDAPOpDtor)(LDAPOp *op);
 
@@ -370,23 +360,21 @@ book_view_notify_status (EDataBookView *view, const char *status)
 static EDataBookView*
 find_book_view (EBookBackendLDAP *bl)
 {
-#if 0
-	EIterator *iter = e_list_get_iterator (bl->priv->book_views);
+	EList *views = e_book_backend_get_book_views (E_BOOK_BACKEND (bl));
+	EIterator *iter = e_list_get_iterator (views);
 	EDataBookView *rv = NULL;
 
 	if (e_iterator_is_valid (iter)) {
 		/* just always use the first book view */
-		EBookBackendLDAPBookView *v = (EBookBackendLDAPBookView*)e_iterator_get(iter);
+		EDataBookView *v = (EDataBookView*)e_iterator_get(iter);
 		if (v)
-			rv = v->book_view;
+			rv = v;
 	}
 
 	g_object_unref (iter);
+	g_object_unref (views);
 
 	return rv;
-#else
-	return NULL;
-#endif
 }
 
 static void
@@ -967,7 +955,7 @@ build_mods_from_contacts (EBookBackendLDAP *bl, EContact *current, EContact *new
 			new_prop_present = (new_prop != NULL);
 		}
 		else {
-			new_prop_bers = prop_info[i].ber_func (new);
+			new_prop_bers = prop_info[i].ber_func ? prop_info[i].ber_func (new) : NULL;
 			new_prop_present = (new_prop_bers != NULL);
 		}
 
@@ -994,11 +982,11 @@ build_mods_from_contacts (EBookBackendLDAP *bl, EContact *current, EContact *new
 				if (new_prop && current_prop)
 					include = *new_prop && strcmp (new_prop, current_prop);
 				else
-					include = (!!new_prop != !!current_prop);
+					include = (new_prop != current_prop) && (!new_prop || *new_prop); /* empty strings cause problems */
 			}
 			else {
 				int j;
-				struct berval **current_prop_bers = prop_info[i].ber_func (current);
+				struct berval **current_prop_bers = prop_info[i].ber_func ? prop_info[i].ber_func (current) : NULL;
 
 				current_prop_present = (current_prop_bers != NULL);
 
@@ -1011,7 +999,7 @@ build_mods_from_contacts (EBookBackendLDAP *bl, EContact *current, EContact *new
 					g_free (current_prop_bers);
 				}
 
-				include = !prop_info[i].compare_func (new, current);
+				include = prop_info[i].compare_func ? !prop_info[i].compare_func (new, current) : FALSE;
 			}
 		}
 
@@ -1547,8 +1535,6 @@ e_book_backend_ldap_modify_contact (EBookBackend *backend,
 
 	ldap = bl->priv->ldap;
 
-	book_view_notify_status (book_view, _("Modifying contact from LDAP server..."));
-
 	do {
 		book_view_notify_status (book_view, _("Modifying contact from LDAP server..."));
 
@@ -1747,8 +1733,6 @@ contact_list_dtor (LDAPOp *op)
 {
 	LDAPGetContactListOp *contact_list_op = (LDAPGetContactListOp*)op;
 
-	g_list_foreach (contact_list_op->contacts, (GFunc)g_object_unref, NULL);
-	g_list_free (contact_list_op->contacts);
 	g_free (contact_list_op);
 }
 
@@ -2162,7 +2146,10 @@ category_compare (EContact *contact1, EContact *contact2)
 	categories1 = e_contact_get_const (contact1, E_CONTACT_CATEGORIES);
 	categories2 = e_contact_get_const (contact2, E_CONTACT_CATEGORIES);
 
-	equal = !strcmp (categories1, categories2);
+	if (categories1 && categories2)
+		equal = !strcmp (categories1, categories2);
+	else
+		equal = (categories1 == categories2);
 
 	g_free (categories1);
 	g_free (categories2);
