@@ -5,6 +5,8 @@
 static GThread *worker_thread;
 static GAsyncQueue *to_worker_queue;
 static GAsyncQueue *from_worker_queue;
+static GStaticMutex idle_mutex = G_STATIC_MUTEX_INIT;
+static guint idle_id = -1;
 
 typedef struct _EBookMsg EBookMsg;
 
@@ -29,16 +31,34 @@ worker (gpointer data)
 }
 
 static gboolean
-main_thread_check_for_response (gpointer data)
+main_thread_get_response (gpointer data)
 {
 	EBookMsg *msg;
 
+	g_static_mutex_lock (&idle_mutex);
+
+	idle_id = -1;
+
 	while ((msg = g_async_queue_try_pop (from_worker_queue)) != NULL) {
 		msg->handler (msg);
 		msg->dtor (msg);
 	}
-	
-	return TRUE;
+
+	g_static_mutex_unlock (&idle_mutex);
+
+	return FALSE;
+}
+
+static void
+push_response (gpointer response)
+{
+	g_static_mutex_lock (&idle_mutex);
+
+	g_async_queue_push (from_worker_queue, response);
+	if (idle_id == -1)
+		idle_id = g_idle_add (main_thread_get_response, NULL);
+
+	g_static_mutex_unlock (&idle_mutex);
 }
 
 static void
@@ -57,12 +77,100 @@ init_async()
 		to_worker_queue = g_async_queue_new ();
 		from_worker_queue = g_async_queue_new ();
 		worker_thread = g_thread_create (worker, NULL, FALSE, NULL);
-		g_timeout_add (300, main_thread_check_for_response, NULL);
 	}
 }
 
 
 
+typedef struct {
+	EBookMsg msg;
+
+	EBook *book;
+	EBookCallback open_response;
+	gpointer closure;
+} LoadLocalBookMsg;
+
+typedef struct {
+	EBookMsg msg;
+
+	EBook *book;
+	EBookStatus status;
+	EBookCallback open_response;
+	gpointer closure;
+} LoadLocalBookResponse;
+
+static void
+_load_local_book_response_handler (EBookMsg *msg)
+{
+	LoadLocalBookResponse *resp = (LoadLocalBookResponse*)msg;
+
+	resp->open_response (resp->book, resp->status, resp->closure);
+}
+
+static void
+_load_local_book_response_dtor (EBookMsg *msg)
+{
+	LoadLocalBookResponse *resp = (LoadLocalBookResponse*)msg;
+
+	g_object_unref (resp->book);
+	g_free (resp);
+}
+
+static void
+_load_local_book_handler (EBookMsg *msg)
+{
+	LoadLocalBookMsg *load_msg = (LoadLocalBookMsg *)msg;
+	LoadLocalBookResponse *response;
+	GError *error = NULL;
+
+	response = g_new0 (LoadLocalBookResponse, 1);
+	e_book_msg_init ((EBookMsg*)response, _load_local_book_response_handler, _load_local_book_response_dtor);
+
+	response->status = E_BOOK_ERROR_OK;
+	if (!e_book_load_local_addressbook (load_msg->book, &error)) {
+		response->status = error->code;
+		g_error_free (error);
+	}
+
+	response->book = load_msg->book;
+	response->open_response = load_msg->open_response;
+	response->closure = load_msg->closure;
+
+	push_response (response);
+}
+
+static void
+_load_local_book_dtor (EBookMsg *msg)
+{
+	LoadLocalBookMsg *load_msg = (LoadLocalBookMsg *)msg;
+	
+	g_free (load_msg);
+}
+
+void
+e_book_async_load_local_addressbook (EBook                 *book,
+				     EBookCallback          open_response,
+				     gpointer               closure)
+{
+	LoadLocalBookMsg *msg;
+
+	g_return_if_fail (E_IS_BOOK (book));
+	g_return_if_fail (open_response != NULL);
+
+	init_async ();
+
+	msg = g_new (LoadLocalBookMsg, 1);
+	e_book_msg_init ((EBookMsg*)msg, _load_local_book_handler, _load_local_book_dtor);
+
+	msg->book = g_object_ref (book);
+	msg->open_response = open_response;
+	msg->closure = closure;
+
+	g_async_queue_push (to_worker_queue, msg);
+}
+
+
+
 typedef struct {
 	EBookMsg msg;
 
@@ -105,7 +213,7 @@ _load_source_handler (EBookMsg *msg)
 	LoadSourceResponse *response;
 	GError *error = NULL;
 
-	response = g_new (LoadSourceResponse, 1);
+	response = g_new0 (LoadSourceResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _load_source_response_handler, _load_source_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -118,7 +226,7 @@ _load_source_handler (EBookMsg *msg)
 	response->open_response = source_msg->open_response;
 	response->closure = source_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 static void
@@ -138,6 +246,10 @@ e_book_async_load_source (EBook                 *book,
 {
 	LoadSourceMsg *msg;
 
+	g_return_if_fail (E_IS_BOOK (book));
+	g_return_if_fail (E_IS_SOURCE (source));
+	g_return_if_fail (open_response != NULL);
+
 	init_async ();
 
 	msg = g_new (LoadSourceMsg, 1);
@@ -161,6 +273,10 @@ e_book_async_load_uri (EBook                 *book,
  	ESourceGroup *group;
 	ESource *source;
 
+	g_return_if_fail (E_IS_BOOK (book));
+	g_return_if_fail (uri != NULL);
+	g_return_if_fail (open_response != NULL);
+
 	init_async ();
 
 	msg = g_new (LoadSourceMsg, 1);
@@ -182,15 +298,8 @@ e_book_async_load_uri (EBook                 *book,
 }
 
 
-void
-e_book_async_unload_uri (EBook                 *book)
-{
-	e_book_unload_uri (book, NULL);
-}
-
 
 
-
 typedef struct {
 	EBookMsg msg;
 
@@ -231,7 +340,7 @@ _default_book_handler (EBookMsg *msg)
 	DefaultBookResponse *response;
 	GError *error = NULL;
 
-	response = g_new (DefaultBookResponse, 1);
+	response = g_new0 (DefaultBookResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _default_book_response_handler, _default_book_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -243,7 +352,7 @@ _default_book_handler (EBookMsg *msg)
 	response->open_response = dfb_msg->open_response;
 	response->closure = dfb_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 static void
@@ -260,6 +369,8 @@ e_book_async_get_default_addressbook (EBookCallback open_response,
 {
 	DefaultBookMsg *msg;
 
+	g_return_if_fail (open_response != NULL);
+
 	init_async ();
 
 	msg = g_new (DefaultBookMsg, 1);
@@ -327,7 +438,7 @@ _get_fields_handler (EBookMsg *msg)
 	GetFieldsResponse *response;
 	GError *error = NULL;
 
-	response = g_new (GetFieldsResponse, 1);
+	response = g_new0 (GetFieldsResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _get_fields_response_handler, _get_fields_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -339,7 +450,7 @@ _get_fields_handler (EBookMsg *msg)
 	response->cb = fields_msg->cb;
 	response->closure = fields_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 guint
@@ -349,6 +460,8 @@ e_book_async_get_supported_fields (EBook                 *book,
 {
 	GetFieldsMsg *msg;
 
+	g_return_val_if_fail (E_IS_BOOK (book), 0);
+
 	init_async ();
 
 	msg = g_new (GetFieldsMsg, 1);
@@ -419,7 +532,7 @@ _get_methods_handler (EBookMsg *msg)
 	GetMethodsResponse *response;
 	GError *error = NULL;
 
-	response = g_new (GetMethodsResponse, 1);
+	response = g_new0 (GetMethodsResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _get_methods_response_handler, _get_methods_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -432,7 +545,7 @@ _get_methods_handler (EBookMsg *msg)
 	response->cb = methods_msg->cb;
 	response->closure = methods_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 guint
@@ -442,6 +555,8 @@ e_book_async_get_supported_auth_methods (EBook                    *book,
 {
 	GetMethodsMsg *msg;
 
+	g_return_val_if_fail (E_IS_BOOK (book), 0);
+
 	init_async ();
 
 	msg = g_new (GetMethodsMsg, 1);
@@ -503,7 +618,7 @@ _auth_user_handler (EBookMsg *msg)
 	AuthUserResponse *response;
 	GError *error = NULL;
 
-	response = g_new (AuthUserResponse, 1);
+	response = g_new0 (AuthUserResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _auth_user_response_handler, _auth_user_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -515,7 +630,7 @@ _auth_user_handler (EBookMsg *msg)
 	response->cb = auth_msg->cb;
 	response->closure = auth_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 static void
@@ -541,6 +656,11 @@ e_book_async_authenticate_user (EBook                 *book,
 {
 	AuthUserMsg *msg;
 
+	g_return_if_fail (E_IS_BOOK (book));
+	g_return_if_fail (user != NULL);
+	g_return_if_fail (passwd != NULL);
+	g_return_if_fail (auth_method != NULL);
+
 	init_async ();
 
 	msg = g_new (AuthUserMsg, 1);
@@ -603,7 +723,7 @@ _get_contact_handler (EBookMsg *msg)
 	GetContactResponse *response;
 	GError *error = NULL;
 
-	response = g_new (GetContactResponse, 1);
+	response = g_new0 (GetContactResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _get_contact_response_handler, _get_contact_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -615,7 +735,7 @@ _get_contact_handler (EBookMsg *msg)
 	response->cb = get_contact_msg->cb;
 	response->closure = get_contact_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 static void
@@ -636,6 +756,9 @@ e_book_async_get_contact (EBook                 *book,
 {
 	GetContactMsg *msg;
 
+	g_return_val_if_fail (E_IS_BOOK (book), 0);
+	g_return_val_if_fail (id != NULL, 0);
+
 	init_async ();
 
 	msg = g_new (GetContactMsg, 1);
@@ -660,18 +783,28 @@ e_book_async_remove_contact (EBook                 *book,
 			     EBookCallback          cb,
 			     gpointer               closure)
 {
-	const char *id = e_contact_get_const (contact, E_CONTACT_UID);
+	const char *id;
+
+	g_return_val_if_fail (E_IS_BOOK (book), FALSE);
+	g_return_val_if_fail (E_IS_CONTACT (contact), FALSE);
+
+	id = e_contact_get_const (contact, E_CONTACT_UID);
 
 	return e_book_async_remove_contact_by_id (book, id, cb, closure);
 }
 
 gboolean
 e_book_async_remove_contact_by_id (EBook                 *book,
-				const char            *id,
-				EBookCallback          cb,
-				gpointer               closure)
+				   const char            *id,
+				   EBookCallback          cb,
+				   gpointer               closure)
 {
-	GList *list = g_list_append (NULL, g_strdup (id));
+	GList *list;
+
+	g_return_val_if_fail (E_IS_BOOK (book), FALSE);
+	g_return_val_if_fail (id != NULL, FALSE);
+
+	list = g_list_append (NULL, g_strdup (id));
 
 	return e_book_async_remove_contacts (book, list, cb, closure);
 }
@@ -720,7 +853,7 @@ _remove_contacts_handler (EBookMsg *msg)
 	RemoveContactsResponse *response;
 	GError *error = NULL;
 
-	response = g_new (RemoveContactsResponse, 1);
+	response = g_new0 (RemoveContactsResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _remove_contacts_response_handler, _remove_contacts_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -732,7 +865,7 @@ _remove_contacts_handler (EBookMsg *msg)
 	response->cb = remove_contacts_msg->cb;
 	response->closure = remove_contacts_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 static void
@@ -747,12 +880,15 @@ _remove_contacts_dtor (EBookMsg *msg)
 
 gboolean
 e_book_async_remove_contacts (EBook                 *book,
-			   GList                 *id_list,
-			   EBookCallback          cb,
-			   gpointer               closure)
+			      GList                 *id_list,
+			      EBookCallback          cb,
+			      gpointer               closure)
 {
 	RemoveContactsMsg *msg;
 	GList *l;
+
+	g_return_val_if_fail (E_IS_BOOK (book), FALSE);
+
 	init_async ();
 
 	msg = g_new (RemoveContactsMsg, 1);
@@ -818,7 +954,7 @@ _add_contact_handler (EBookMsg *msg)
 	AddContactResponse *response;
 	GError *error = NULL;
 
-	response = g_new (AddContactResponse, 1);
+	response = g_new0 (AddContactResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _add_contact_response_handler, _add_contact_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -834,7 +970,7 @@ _add_contact_handler (EBookMsg *msg)
 	response->cb = add_contact_msg->cb;
 	response->closure = add_contact_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 static void
@@ -854,6 +990,9 @@ e_book_async_add_contact (EBook                 *book,
 {
 	AddContactMsg *msg;
 
+	g_return_val_if_fail (E_IS_BOOK (book), FALSE);
+	g_return_val_if_fail (E_IS_CONTACT (contact), FALSE);
+
 	init_async ();
 
 	msg = g_new (AddContactMsg, 1);
@@ -915,7 +1054,7 @@ _commit_contact_handler (EBookMsg *msg)
 	CommitContactResponse *response;
 	GError *error = NULL;
 
-	response = g_new (CommitContactResponse, 1);
+	response = g_new0 (CommitContactResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _commit_contact_response_handler, _commit_contact_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -927,7 +1066,7 @@ _commit_contact_handler (EBookMsg *msg)
 	response->cb = commit_contact_msg->cb;
 	response->closure = commit_contact_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 static void
@@ -948,6 +1087,9 @@ e_book_async_commit_contact (EBook                 *book,
 {
 	CommitContactMsg *msg;
 
+	g_return_val_if_fail (E_IS_BOOK (book), FALSE);
+	g_return_val_if_fail (E_IS_CONTACT (contact), FALSE);
+
 	init_async ();
 
 	msg = g_new (CommitContactMsg, 1);
@@ -1013,7 +1155,7 @@ _get_book_view_handler (EBookMsg *msg)
 	GetBookViewResponse *response;
 	GError *error = NULL;
 
-	response = g_new (GetBookViewResponse, 1);
+	response = g_new0 (GetBookViewResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _get_book_view_response_handler, _get_book_view_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -1025,7 +1167,7 @@ _get_book_view_handler (EBookMsg *msg)
 	response->cb = view_msg->cb;
 	response->closure = view_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 static void
@@ -1049,6 +1191,9 @@ e_book_async_get_book_view (EBook                 *book,
 	GetBookViewMsg *msg;
 	GList *l;
 
+	g_return_val_if_fail (E_IS_BOOK (book), 0);
+	g_return_val_if_fail (query != NULL, 0);
+
 	init_async ();
 
 	msg = g_new (GetBookViewMsg, 1);
@@ -1114,7 +1259,7 @@ _get_contacts_handler (EBookMsg *msg)
 	GetContactsResponse *response;
 	GError *error = NULL;
 
-	response = g_new (GetContactsResponse, 1);
+	response = g_new0 (GetContactsResponse, 1);
 	e_book_msg_init ((EBookMsg*)response, _get_contacts_response_handler, _get_contacts_response_dtor);
 
 	response->status = E_BOOK_ERROR_OK;
@@ -1126,7 +1271,7 @@ _get_contacts_handler (EBookMsg *msg)
 	response->cb = view_msg->cb;
 	response->closure = view_msg->closure;
 
-	g_async_queue_push (from_worker_queue, response);
+	push_response (response);
 }
 
 static void
@@ -1146,6 +1291,9 @@ e_book_async_get_contacts (EBook                 *book,
 {
 	GetContactsMsg *msg;
 
+	g_return_val_if_fail (E_IS_BOOK (book), 0);
+	g_return_val_if_fail (query != NULL, 0);
+
 	init_async ();
 
 	msg = g_new (GetContactsMsg, 1);
