@@ -153,7 +153,6 @@ e_book_get_op (EBook *book)
 static void
 e_book_op_free (EBookOp *op)
 {
-	/* XXX more stuff here */
 	g_cond_free (op->cond);
 	g_mutex_free (op->mutex);
 	g_free (op);
@@ -1340,6 +1339,8 @@ e_book_cancel (EBook   *book,
 	gboolean rv;
 	CORBA_Environment ev;
 
+	e_return_error_if_fail (book && E_IS_BOOK (book),       E_BOOK_ERROR_INVALID_ARG);
+
 	g_mutex_lock (book->priv->mutex);
 
 	if (book->priv->current_op == NULL) {
@@ -1601,7 +1602,7 @@ e_book_handle_response (EBookListener *listener, EBookListenerResponse *resp, EB
  *
  * Return value: #TRUE on success, #FALSE otherwise.
  */
-gboolean
+static gboolean
 e_book_unload_uri (EBook   *book,
 		   GError **error)
 {
@@ -1610,26 +1611,31 @@ e_book_unload_uri (EBook   *book,
 	e_return_error_if_fail (book && E_IS_BOOK (book), E_BOOK_ERROR_INVALID_ARG);
 	e_return_error_if_fail (book->priv->load_state != E_BOOK_URI_NOT_LOADED, E_BOOK_ERROR_URI_NOT_LOADED);
 
-	/* Release the remote GNOME_Evolution_Addressbook_Book in the PAS. */
-	CORBA_exception_init (&ev);
+	if (book->priv->load_state == E_BOOK_URI_LOADED) {
+		/* Release the remote GNOME_Evolution_Addressbook_Book in the PAS. */
+		CORBA_exception_init (&ev);
 
-	bonobo_object_release_unref  (book->priv->corba_book, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION) {
-		g_warning ("e_book_unload_uri: Exception releasing "
-			   "remote book interface!\n");
+		bonobo_object_release_unref  (book->priv->corba_book, &ev);
+		if (ev._major != CORBA_NO_EXCEPTION) {
+			g_warning ("e_book_unload_uri: Exception releasing "
+				   "remote book interface!\n");
+		}
+
+		CORBA_exception_free (&ev);
+
+		e_book_listener_stop (book->priv->listener);
+		bonobo_object_unref (BONOBO_OBJECT (book->priv->listener));
+
+		book->priv->listener   = NULL;
+		book->priv->load_state = E_BOOK_URI_NOT_LOADED;
+		g_free (book->priv->cap);
+		book->priv->cap = NULL;
+		book->priv->cap_queried = FALSE;
+		book->priv->writable = FALSE;
+	}
+	else if (book->priv->load_state == E_BOOK_URI_LOADING) {
+		e_book_cancel (book, error);
 	}
-
-	CORBA_exception_free (&ev);
-
-	e_book_listener_stop (book->priv->listener);
-	bonobo_object_unref (BONOBO_OBJECT (book->priv->listener));
-
-	book->priv->listener   = NULL;
-	book->priv->load_state = E_BOOK_URI_NOT_LOADED;
-	g_free (book->priv->cap);
-	book->priv->cap = NULL;
-	book->priv->cap_queried = FALSE;
-	book->priv->writable = FALSE;
 
 	return TRUE;
 }
@@ -1674,7 +1680,9 @@ activate_factories_for_uri (EBook *book, const char *uri)
 	info_list = bonobo_activation_query (query, NULL, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
-		g_warning ("Eeek!  Cannot perform bonobo-activation query for book factories.");
+		char *exc_text = bonobo_exception_get_text (&ev);
+		g_warning ("Cannot perform bonobo-activation query for book factories: %s", exc_text);
+		g_free (exc_text);
 		CORBA_exception_free (&ev);
 		goto done;
 		return NULL;
@@ -1712,6 +1720,9 @@ activate_factories_for_uri (EBook *book, const char *uri)
 	return factories;
 }
 
+/* XXX hm, should this function hold a lock on book->priv->mutex?  It
+   doesn't seem to require it, but there are unlocked writes to
+   book->priv->load_state, which other functions read.. */
 static gboolean
 fetch_corba_book (EBook       *book,
 		  ESource     *source,
@@ -1723,12 +1734,14 @@ fetch_corba_book (EBook       *book,
 	gchar *source_xml;
 	GList *factories;
 	GList *l;
+	EBookStatus status = E_BOOK_ERROR_OTHER_ERROR;
 	gboolean rv = FALSE;
 
 	uri = e_source_get_uri (source);
 	if (!uri) {
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_OTHER_ERROR,
 			     _("e_book_load_uri: Invalid source."));
+		book->priv->load_state = E_BOOK_URI_NOT_LOADED;
 		return FALSE;
 	}
 
@@ -1737,12 +1750,11 @@ fetch_corba_book (EBook       *book,
 	if (!factories) {
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_PROTOCOL_NOT_SUPPORTED,
 			     _("e_book_load_uri: no factories available for uri `%s'"), uri);
+		book->priv->load_state = E_BOOK_URI_NOT_LOADED;
 		return FALSE;
 	}
 
 
-	book->priv->load_state = E_BOOK_URI_LOADING;
-
 	/*
 	 * Create our local BookListener interface.
 	 */
@@ -1751,6 +1763,7 @@ fetch_corba_book (EBook       *book,
 		g_warning ("e_book_load_uri: Could not create EBookListener!\n");
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_OTHER_ERROR,
 			     _("e_book_load_uri: Could not create EBookListener"));
+		book->priv->load_state = E_BOOK_URI_NOT_LOADED;
 		return FALSE;
 	}
 	book->priv->listener_signal = g_signal_connect (book->priv->listener, "response",
@@ -1770,8 +1783,12 @@ fetch_corba_book (EBook       *book,
 		GNOME_Evolution_Addressbook_BookFactory factory = l->data;
 		EBookOp *our_op;
 		CORBA_Environment ev;
-		EBookStatus status;
 
+		/* we don't bother locking the book's mutex here
+		   before creating the op since there should be no way
+		   another thread could get to a place where they
+		   could add an op (as the load_state !=
+		   E_BOOK_URI_LOADED) */
 		our_op = e_book_new_op (book);
 
 		g_mutex_lock (our_op->mutex);
@@ -1830,18 +1847,27 @@ fetch_corba_book (EBook       *book,
 		CORBA_Object_release ((CORBA_Object)l->data, NULL);
 
 	if (rv == TRUE) {
-		book->priv->corba_book = corba_book;
-		book->priv->load_state = E_BOOK_URI_LOADED;
-		book->priv->comp_listener = e_component_listener_new (book->priv->corba_book);
-		book->priv->died_signal = g_signal_connect (book->priv->comp_listener, "component_died",
-							    G_CALLBACK (backend_died_cb), book);
-		return TRUE;
+		if (status == E_BOOK_ERROR_OK) {
+			book->priv->corba_book = corba_book;
+			book->priv->load_state = E_BOOK_URI_LOADED;
+			book->priv->comp_listener = e_component_listener_new (book->priv->corba_book);
+			book->priv->died_signal = g_signal_connect (book->priv->comp_listener,
+								    "component_died",
+								    G_CALLBACK (backend_died_cb), book);
+		} else {
+			/* Cancelled */
+			book->priv->load_state = E_BOOK_URI_NOT_LOADED;
+			g_signal_handler_disconnect (book->priv->listener, book->priv->listener_signal);
+			bonobo_object_unref (book->priv->listener);
+			book->priv->listener = NULL;
+			g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_CANCELLED,
+				     _("e_book_load_uri: cancelled"));
+		}
 	}
 	else {
 		book->priv->load_state = E_BOOK_URI_NOT_LOADED;
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_PROTOCOL_NOT_SUPPORTED,
 			     _("e_book_load_uri: no factories available for uri `%s'"), uri);
-		return FALSE;
 	}
 		
 	return rv;
@@ -1867,8 +1893,23 @@ e_book_load_source (EBook *book,
 	e_return_error_if_fail (book && E_IS_BOOK (book),       E_BOOK_ERROR_INVALID_ARG);
 	e_return_error_if_fail (source && E_IS_SOURCE (source), E_BOOK_ERROR_INVALID_ARG);
 
-	/* XXX this needs to happen while holding the book's lock i would think... */
-	e_return_error_if_fail (book->priv->load_state == E_BOOK_URI_NOT_LOADED, E_BOOK_ERROR_URI_ALREADY_LOADED);
+	g_mutex_lock (book->priv->mutex);
+
+	if (book->priv->load_state != E_BOOK_URI_NOT_LOADED) {
+		g_mutex_unlock (book->priv->mutex);
+		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_URI_ALREADY_LOADED,
+			     _("\"%s\" on book after \"%s\""),
+			     "e_book_load_source", "e_book_load_source");
+		return FALSE;
+	}
+
+	/* we have to set this here so that future calls to
+	   e_book_load* will fail.  It will be cleared to
+	   E_BOOK_URI_NOT_LOADED in fetch_corba_book if there's an
+	   error. */
+	book->priv->load_state = E_BOOK_URI_LOADING;
+
+	g_mutex_unlock (book->priv->mutex);
 
 	return fetch_corba_book (book, source, only_if_exists, error);
 }
@@ -1897,9 +1938,6 @@ e_book_load_uri (EBook        *book,
 	e_return_error_if_fail (book && E_IS_BOOK (book), E_BOOK_ERROR_INVALID_ARG);
 	e_return_error_if_fail (uri,                      E_BOOK_ERROR_INVALID_ARG);
 
-	/* XXX this needs to happen while holding the book's lock i would think... */
-	e_return_error_if_fail (book->priv->load_state == E_BOOK_URI_NOT_LOADED, E_BOOK_ERROR_URI_ALREADY_LOADED);
-
 	group = e_source_group_new ("", uri);
 	source = e_source_new ("", "");
 	e_source_set_group (source, group);
@@ -1929,14 +1967,16 @@ e_book_load_local_addressbook (EBook   *book,
 	char *uri;
 	gboolean rv;
 
+	e_return_error_if_fail (book && E_IS_BOOK (book),       E_BOOK_ERROR_INVALID_ARG);
+
 	filename = g_build_filename (g_get_home_dir(),
-				     ".evolution/addressbook/local/OnThisComputer/Personal",
+				     ".evolution/addressbook/local/system",
 				     NULL);
 	uri = g_strdup_printf ("file://%s", filename);
 
 	g_free (filename);
 	
-	rv = e_book_load_uri (book, uri, TRUE, error);
+	rv = e_book_load_uri (book, uri, FALSE, error);
 	
 	g_free (uri);
 
@@ -1954,6 +1994,8 @@ e_book_load_local_addressbook (EBook   *book,
 const char *
 e_book_get_uri (EBook *book)
 {
+	g_return_val_if_fail (book && E_IS_BOOK (book), NULL);
+
 	return book->priv->uri;
 }
 
@@ -1968,6 +2010,8 @@ e_book_get_uri (EBook *book)
 ESource *
 e_book_get_source (EBook *book)
 {
+	g_return_val_if_fail (book && E_IS_BOOK (book), NULL);
+
 	return book->priv->source;
 }
 
@@ -1985,6 +2029,8 @@ const char *
 e_book_get_static_capabilities (EBook   *book,
 				GError **error)
 {
+	g_return_val_if_fail (book && E_IS_BOOK (book), NULL);
+
 	if (!book->priv->cap_queried) {
 		CORBA_Environment ev;
 		char *temp;
@@ -1992,17 +2038,20 @@ e_book_get_static_capabilities (EBook   *book,
 		CORBA_exception_init (&ev);
 
 		if (book->priv->load_state != E_BOOK_URI_LOADED) {
-			g_warning ("e_book_unload_uri: No URI is loaded!\n");
-			return g_strdup("");
+			g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_URI_NOT_LOADED,
+				     _("\"%s\" on book before \"%s\""),
+				     "e_book_get_static_capabilities", "e_book_load_uri");
+			return g_strdup ("");
 		}
 
 		temp = GNOME_Evolution_Addressbook_Book_getStaticCapabilities(book->priv->corba_book, &ev);
 
 		if (ev._major != CORBA_NO_EXCEPTION) {
-			g_warning ("e_book_get_static_capabilities: Exception "
-				   "during get_static_capabilities!\n");
+			g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_CORBA_EXCEPTION,
+				     _("CORBA exception making \"%s\" call"),
+				     "Book::getStaticCapabilities");
 			CORBA_exception_free (&ev);
-			return g_strdup("");
+			return g_strdup ("");
 		}
 
 		book->priv->cap = g_strdup(temp);
@@ -2030,7 +2079,11 @@ gboolean
 e_book_check_static_capability (EBook *book,
 				const char  *cap)
 {
-	const char *caps = e_book_get_static_capabilities (book, NULL);
+	const char *caps;
+
+	g_return_val_if_fail (book && E_IS_BOOK (book), FALSE);
+
+	caps = e_book_get_static_capabilities (book, NULL);
 
 	/* XXX this is an inexact test but it works for our use */
 	if (caps && strstr (caps, cap))
@@ -2050,6 +2103,8 @@ e_book_check_static_capability (EBook *book,
 gboolean
 e_book_is_writable (EBook *book)
 {
+	g_return_val_if_fail (book && E_IS_BOOK (book), FALSE);
+
 	return book->priv->writable;
 }
 
@@ -2096,11 +2151,15 @@ e_book_get_self (EContact **contact, EBook **book, GError **error)
 gboolean
 e_book_set_self (EBook *book, EContact *contact, GError **error)
 {
-	GConfClient *gconf = gconf_client_get_default();
+	GConfClient *gconf;
 
+	e_return_error_if_fail (book && E_IS_BOOK (book),          E_BOOK_ERROR_INVALID_ARG);
+	e_return_error_if_fail (contact && E_IS_CONTACT (contact), E_BOOK_ERROR_INVALID_ARG);
+
+	gconf = gconf_client_get_default();
 	gconf_client_set_string (gconf, SELF_UID_KEY, e_contact_get_const (contact, E_CONTACT_UID), NULL);
-
 	g_object_unref (gconf);
+
 	return TRUE;
 }
 
@@ -2111,6 +2170,10 @@ e_book_is_self (EContact *contact)
 	char *uid;
 	gboolean rv;
 
+	/* XXX this should probably be e_return_error_if_fail, but we
+	   need a GError** arg for that */
+	g_return_val_if_fail (contact && E_IS_CONTACT (contact), FALSE);
+
 	gconf = gconf_client_get_default();
 	uid = gconf_client_get_string (gconf, SELF_UID_KEY, NULL);
 	g_object_unref (gconf);
@@ -2137,32 +2200,143 @@ e_book_is_self (EContact *contact)
 gboolean
 e_book_get_default_addressbook (EBook **book, GError **error)
 {
-	/* XXX for now just load the local ~/evolution/local/Contacts,
-	   this should really use properties on the ESource's that
-	   represent the addressbooks we have available. */
-	char *path, *uri;
-	gboolean rv;
+	ESourceList *sources;
+	GSList *g;
+	GError *err = NULL;
+	ESource *default_source = NULL;
+	gboolean rv = TRUE;
+
+	e_return_error_if_fail (*book, E_BOOK_ERROR_INVALID_ARG);
+
+	if (!e_book_get_addressbooks (&sources, &err)) {
+		g_propagate_error (error, err);
+		return FALSE;
+	}
+
+	for (g = e_source_list_peek_groups (sources); g; g = g->next) {
+		ESourceGroup *group = E_SOURCE_GROUP (g->data);
+		GSList *s;
+		for (s = e_source_group_peek_sources (group); s; s = s->next) {
+			ESource *source = E_SOURCE (s->data);
+
+			if (e_source_get_property (source, "default")) {
+				default_source = source;
+				break;
+			}
+		}
+
+		if (default_source)
+			break;
+	}
 
 	*book = e_book_new ();
 
-	path = g_build_filename (g_get_home_dir (),
-				 ".evolution/addressbook/local/OnThisComputer/Personal",
-				 NULL);
-	uri = g_strdup_printf ("file://%s", path);
-	g_free (path);
-
-	rv = e_book_load_uri (*book, uri, FALSE, error);
-
-	g_free (uri);
+	if (default_source) {
+		if (!e_book_load_source (*book, default_source, TRUE, &err)) {
+			g_propagate_error (error, err);
+			rv = FALSE;
+			goto done;
+		}
+	}
+	else {
+		if (!e_book_load_local_addressbook (*book, &err)) {
+			g_propagate_error (error, err);
+			rv = FALSE;
+			goto done;
+		}
+	}
 
+ done:
 	if (!rv) {
 		g_object_unref (*book);
 		*book = NULL;
 	}
-
+	g_object_unref (sources);
 	return rv;
 }
 
+/**
+ * e_book_set_default_addressbook:
+ * @book: An #EBook pointer
+ * @error: A #GError pointer
+ * 
+ * sets the #ESource of the #EBook as the "default" addressbook.  This is the source
+ * that will be loaded in the e_book_get_default_addressbook call.
+ * 
+ * Return value: #TRUE if the setting was stored in libebook's ESourceList, otherwise #FALSE.
+ */
+gboolean
+e_book_set_default_addressbook (EBook *book, GError **error)
+{
+	ESource *source;
+
+	e_return_error_if_fail (book && E_IS_BOOK (book),                        E_BOOK_ERROR_INVALID_ARG);
+	e_return_error_if_fail (book->priv->load_state == E_BOOK_URI_NOT_LOADED, E_BOOK_ERROR_URI_ALREADY_LOADED);
+
+	source = e_book_get_source (book);
+
+	return e_book_set_default_source (source, error);
+}
+
+
+/**
+ * e_book_set_default_addressbook:
+ * @source: An #ESource pointer
+ * @error: A #GError pointer
+ * 
+ * sets @source as the "default" addressbook.  This is the source that
+ * will be loaded in the e_book_get_default_addressbook call.
+ * 
+ * Return value: #TRUE if the setting was stored in libebook's ESourceList, otherwise #FALSE.
+ */
+gboolean
+e_book_set_default_source (ESource *source, GError **error)
+{
+	ESourceList *sources;
+	const char *uid;
+	GError *err = NULL;
+	GSList *g;
+
+	e_return_error_if_fail (source && E_IS_SOURCE (source), E_BOOK_ERROR_INVALID_ARG);
+
+	uid = e_source_peek_uid (source);
+
+	if (!e_book_get_addressbooks (&sources, &err)) {
+		g_propagate_error (error, err);
+		return FALSE;
+	}
+
+	/* make sure the source is actually in the ESourceList.  if
+	   it's not we don't bother adding it, just return an error */
+	source = e_source_list_peek_source_by_uid (sources, uid);
+	if (!source) {
+		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_NO_SUCH_SOURCE,
+			     _("e_book_set_default_source: there was no source for uid `%s' stored in gconf."), uid);
+		g_object_unref (sources);
+		return FALSE;
+	}
+
+	/* loop over all the sources clearing out any "default"
+	   properties we find */
+	for (g = e_source_list_peek_groups (sources); g; g = g->next) {
+		GSList *s;
+		for (s = e_source_group_peek_sources (E_SOURCE_GROUP (g->data));
+		     s; s = s->next) {
+			e_source_set_property (E_SOURCE (s->data), "default", NULL);
+		}
+	}
+
+	/* set the "default" property on the source */
+	e_source_set_property (source, "default", "true");
+
+	if (!e_source_list_sync (sources, &err)) {
+		g_propagate_error (error, err);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 /**
  * e_book_get_addressbooks:
  * @addressbook_sources: A pointer to a ESourceList* to set
@@ -2176,8 +2350,11 @@ e_book_get_default_addressbook (EBook **book, GError **error)
 gboolean
 e_book_get_addressbooks (ESourceList **addressbook_sources, GError **error)
 {
-	GConfClient *gconf = gconf_client_get_default();
+	GConfClient *gconf;
 
+	e_return_error_if_fail (*addressbook_sources, E_BOOK_ERROR_INVALID_ARG);
+
+	gconf = gconf_client_get_default();
 	*addressbook_sources = e_source_list_new_for_gconf (gconf, "/apps/evolution/addressbook/sources");
 	g_object_unref (gconf);
 
@@ -2188,7 +2365,8 @@ e_book_get_addressbooks (ESourceList **addressbook_sources, GError **error)
 static void*
 startup_mainloop (void *arg)
 {
-	bonobo_main();
+	GMainLoop *loop = g_main_loop_new (NULL, FALSE);
+	g_main_loop_run (loop);
 	return NULL;
 }
 
@@ -2203,6 +2381,10 @@ e_book_activate()
 	if (!activated) {
 		pthread_t ebook_mainloop_thread;
 		activated = TRUE;
+
+		if (!bonobo_is_initialized ())
+			bonobo_init (NULL, NULL);
+
 		pthread_create(&ebook_mainloop_thread, NULL, startup_mainloop, NULL);
 	}
 	g_static_mutex_unlock (&e_book_lock);
