@@ -37,6 +37,7 @@ typedef struct _EBookPrivate EBookPrivate;
 
 struct _EBook {
 	GObject       parent;
+	/*< private >*/
 	EBookPrivate *priv;
 };
 
@@ -71,9 +72,6 @@ gboolean e_book_load_uri                   (EBook       *book,
 					    gboolean     only_if_exists,
 					    GError     **error);
 
-gboolean e_book_unload_uri                 (EBook       *book,
-					    GError     **error);
-
 gboolean e_book_remove                     (EBook       *book,
 					    GError     **error);
 
@@ -108,7 +106,7 @@ gboolean e_book_remove_contact             (EBook       *book,
 					    GError     **error);
 
 gboolean e_book_remove_contacts            (EBook       *book,
-					    GList       *id_list,
+					    GList       *ids,
 					    GError     **error);
 
 /* Adding contacts. */
@@ -143,6 +141,7 @@ gboolean e_book_get_changes                (EBook       *book,
 void     e_book_free_change_list           (GList       *change_list);
 
 const char *e_book_get_uri                 (EBook       *book);
+ESource    *e_book_get_source              (EBook       *book);
 
 const char *e_book_get_static_capabilities (EBook    *book,
 					    GError  **error);
@@ -161,6 +160,8 @@ gboolean    e_book_is_self                 (EContact *contact);
 
 /* Addressbook Discovery */
 gboolean    e_book_get_default_addressbook (EBook **book, GError **error);
+gboolean    e_book_set_default_addressbook (EBook  *book, GError **error);
+gboolean    e_book_set_default_source      (ESource *source, GError **error);
 gboolean    e_book_get_addressbooks        (ESourceList** addressbook_sources, GError **error);
 
 GType        e_book_get_type                  (void);
