@@ -83,7 +83,7 @@ static void cert_setter (EContact *contact, EVCardAttribute *attr, void *data);
 #define BOOLEAN_FIELD(id,vc,n,pn,ro)  { E_CONTACT_FIELD_TYPE_BOOLEAN, (id), (vc), (n), (pn), (ro) }
 #define LIST_FIELD(id,vc,n,pn,ro)      { E_CONTACT_FIELD_TYPE_LIST, (id), (vc), (n), (pn), (ro) }
 #define MULTI_LIST_FIELD(id,vc,n,pn,ro) { E_CONTACT_FIELD_TYPE_MULTI, (id), (vc), (n), (pn), (ro) }
-#define GETSET_FIELD(id,vc,n,pn,ro,get,set)    { E_CONTACT_FIELD_TYPE_GETSET, (id), (vc), (n), (pn), (ro), -1, NULL, NULL, (get), (set) }
+#define GETSET_FIELD(id,vc,n,pn,ro,get,set)    { E_CONTACT_FIELD_TYPE_STRING | E_CONTACT_FIELD_TYPE_GETSET, (id), (vc), (n), (pn), (ro), -1, NULL, NULL, (get), (set) }
 #define STRUCT_FIELD(id,vc,n,pn,ro,get,set,ty)    { E_CONTACT_FIELD_TYPE_STRUCT | E_CONTACT_FIELD_TYPE_GETSET, (id), (vc), (n), (pn), (ro), -1, NULL, NULL, (get), (set), (ty) }
 #define SYNTH_STR_FIELD(id,n,pn,ro)  { E_CONTACT_FIELD_TYPE_STRING | E_CONTACT_FIELD_TYPE_SYNTHETIC, (id), NULL, (n), (pn), (ro) }
 #define LIST_ELEM_STR_FIELD(id,vc,n,pn,ro,nm) { E_CONTACT_FIELD_TYPE_LIST_ELEM | E_CONTACT_FIELD_TYPE_SYNTHETIC | E_CONTACT_FIELD_TYPE_STRING, (id), (vc), (n), (pn), (ro), (nm) }
@@ -130,8 +130,8 @@ static EContactFieldInfo field_info[] = {
 	ATTR2_TYPE_STR_FIELD (E_CONTACT_PHONE_HOME_FAX,     EVC_TEL, "home_fax",          N_("Home Fax"),         FALSE, "HOME", "FAX",           0),
 	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_ISDN,         EVC_TEL, "isdn_phone",        N_("ISDN"),             FALSE, "ISDN",                  0),
 	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_MOBILE,       EVC_TEL, "mobile_phone",      N_("Mobile Phone"),     FALSE, "CELL",                  0),
-	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_OTHER,        EVC_TEL, "other_phone",       N_("Other Phone"),      FALSE, "VOICE",                 0), /* XXX */
-	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_OTHER_FAX,    EVC_TEL, "other_fax",         N_("Other Fax"),        FALSE, "FAX",                   0), /* XXX */
+	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_OTHER,        EVC_TEL, "other_phone",       N_("Other Phone"),      FALSE, "VOICE",                 0),
+	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_OTHER_FAX,    EVC_TEL, "other_fax",         N_("Other Fax"),        FALSE, "FAX",                   0),
 	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_PAGER,        EVC_TEL, "pager",             N_("Pager"),            FALSE, "PAGER",                 0),
 	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_PRIMARY,      EVC_TEL, "primary_phone",     N_("Primary Phone"),    FALSE, "PREF",                  0),
 	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_RADIO,        EVC_TEL, "radio",             N_("Radio"),            FALSE, EVC_X_RADIO,     0),
@@ -195,9 +195,9 @@ static EContactFieldInfo field_info[] = {
 	STRING_FIELD    (E_CONTACT_ASSISTANT, EVC_X_ASSISTANT, "assistant", N_("Assistant"),       FALSE),
 
 	/* Web fields */
-	STRING_FIELD (E_CONTACT_HOMEPAGE_URL, EVC_URL,        "homepage_url", N_("Homepage URL"), FALSE),
-	STRING_FIELD (E_CONTACT_BLOG_URL,     EVC_X_BLOG_URL, "blog_url",     N_("Weblog URL"),   FALSE),
-	STRING_FIELD (E_CONTACT_VIDEO_URL,     EVC_X_VIDEO_URL, "video_url",     N_("Video Conferencing URL"),   FALSE),
+	STRING_FIELD (E_CONTACT_HOMEPAGE_URL, EVC_URL,         "homepage_url", N_("Homepage URL"), FALSE),
+	STRING_FIELD (E_CONTACT_BLOG_URL,     EVC_X_BLOG_URL,  "blog_url",     N_("Weblog URL"),   FALSE),
+	STRING_FIELD (E_CONTACT_VIDEO_URL,    EVC_X_VIDEO_URL, "video_url",    N_("Video Conferencing URL"),   FALSE),
 
 	/* Photo/Logo */
 	STRUCT_FIELD    (E_CONTACT_PHOTO, EVC_PHOTO, "photo", N_("Photo"), FALSE, photo_getter, photo_setter, e_contact_photo_get_type),
@@ -207,12 +207,8 @@ static EContactFieldInfo field_info[] = {
 	ATTR_TYPE_STRUCT_FIELD (E_CONTACT_X509_CERT,  EVC_KEY, "x509Cert",  N_("X.509 Certificate"), FALSE, "X509", cert_getter, cert_setter, e_contact_cert_get_type),
 
 	/* Contact categories */
-#if notyet
 	LIST_FIELD      (E_CONTACT_CATEGORY_LIST, EVC_CATEGORIES, "category_list", N_("Category List"), FALSE),
 	SYNTH_STR_FIELD (E_CONTACT_CATEGORIES,                    "categories",    N_("Categories"),    FALSE),
-#else
-	STRING_FIELD    (E_CONTACT_CATEGORIES,    EVC_CATEGORIES, "categories",    N_("Categories"),    FALSE),
-#endif
 
 	/* Collaboration fields */
 	STRING_FIELD (E_CONTACT_CALENDAR_URI, EVC_CALURI,      "caluri",     N_("Calendar URI"),  FALSE),
@@ -365,8 +361,12 @@ photo_getter (EContact *contact, EVCardAttribute *attr)
 
 		if (values && values->data) {
 			GString *s = values->data;
-			EContactPhoto *photo = g_new (EContactPhoto, 1);
+			EContactPhoto *photo;
 
+			if (!s->len)
+				return NULL;
+
+			photo = g_new (EContactPhoto, 1);
 			photo->length = s->len;
 			photo->data = g_malloc (photo->length);
 			memcpy (photo->data, s->str, photo->length);
@@ -385,6 +385,11 @@ photo_setter (EContact *contact, EVCardAttribute *attr, void *data)
 	const char *mime_type;
 	char *image_type = "X-EVOLUTION-UNKNOWN";
 
+	if (!photo)
+		return;
+
+	g_return_if_fail (photo->length > 0);
+
 	e_vcard_attribute_add_param_with_value (attr,
 						e_vcard_attribute_param_new (EVC_ENCODING),
 						"b");
@@ -427,6 +432,11 @@ fn_getter (EContact *contact, EVCardAttribute *attr)
 static void
 fn_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
+	gchar *name_str = data;
+
+	if (!name_str)
+		return;
+
 	e_vcard_attribute_add_value (attr, (char*)data);
 
 	attr = e_contact_get_first_attr (contact, EVC_N);
@@ -467,6 +477,9 @@ n_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
 	EContactName *name = data;
 
+	if (!name)
+		return;
+
 	e_vcard_attribute_add_value (attr, name->family);
 	e_vcard_attribute_add_value (attr, name->given);
 	e_vcard_attribute_add_value (attr, name->additional);
@@ -524,6 +537,9 @@ adr_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
 	EContactAddress *addr = data;
 
+	if (!addr)
+		return;
+
 	e_vcard_attribute_add_value (attr, addr->po);
 	e_vcard_attribute_add_value (attr, addr->ext);
 	e_vcard_attribute_add_value (attr, addr->street);
@@ -540,7 +556,12 @@ date_getter (EContact *contact, EVCardAttribute *attr)
 {
 	if (attr) {
 		GList *p = e_vcard_attribute_get_values (attr);
-		EContactDate *date = e_contact_date_from_string (p && p->data ? (char*)p->data : "");
+		EContactDate *date;
+
+		if (p && p->data && ((gchar *) p->data) [0])
+			date = e_contact_date_from_string ((char *) p->data);
+		else
+			date = NULL;
 
 		return date;
 	}
@@ -552,7 +573,12 @@ static void
 date_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
 	EContactDate *date = data;
-	char *str = e_contact_date_to_string (date);
+	char *str;
+
+	if (!date)
+		return;
+
+	str = e_contact_date_to_string (date);
 
 	e_vcard_attribute_add_value (attr, str);
 	g_free (str);
@@ -593,6 +619,9 @@ cert_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
 	EContactCert *cert = data;
 
+	if (!cert)
+		return;
+
 	e_vcard_attribute_add_param_with_value (attr,
 						e_vcard_attribute_param_new (EVC_ENCODING),
 						"b");
@@ -704,15 +733,8 @@ e_contact_set_property (GObject *object,
 				const char *name, *group;
 				gboolean found_needed1, found_needed2;
 
-				if (!info->attr_type1)
-					found_needed1 = TRUE;
-				else
-					found_needed1 = FALSE;
-
-				if (!info->attr_type2)
-					found_needed2 = TRUE;
-				else
-					found_needed2 = FALSE;
+				found_needed1 = (info->attr_type1 == NULL);
+				found_needed2 = (info->attr_type2 == NULL);
 
 				attr = l->data;
 				group = e_vcard_attribute_get_group (attr);
@@ -729,10 +751,24 @@ e_contact_set_property (GObject *object,
 						if (!strcasecmp (name, EVC_TYPE)) {
 							GList *values = e_vcard_attribute_param_get_values (param);
 							if (values && values->data) {
-								if (!found_needed1 && !strcasecmp ((char*)values->data, info->attr_type1))
+								gboolean matches = FALSE;
+								if (!found_needed1 && !strcasecmp ((char*)values->data, info->attr_type1)) {
 									found_needed1 = TRUE;
-								else if (!found_needed2 && !strcasecmp ((char*)values->data, info->attr_type2))
+									matches = TRUE;
+								}
+								else if (!found_needed2 && !strcasecmp ((char*)values->data, info->attr_type2)) {
 									found_needed2 = TRUE;
+									matches = TRUE;
+								}
+								if (!matches) {
+									/* this is to enforce that we find an attribute
+									   with *only* the TYPE='s we need.  This may seem like
+									   an odd restriction but it's the only way at present to
+									   implement the Other Fax and Other Phone attributes. */
+									found_needed1 =
+										found_needed2 = FALSE;
+									break;
+								}
 							}
 						}
 
@@ -763,7 +799,7 @@ e_contact_set_property (GObject *object,
 			}
 
 			if (info->t & E_CONTACT_FIELD_TYPE_STRUCT || info->t & E_CONTACT_FIELD_TYPE_GETSET) {
-				void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : g_value_get_pointer (value);
+				void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : (char*)g_value_get_string (value);
 
 				info->struct_setter (contact, attr, data);
 			}
@@ -810,26 +846,64 @@ e_contact_set_property (GObject *object,
 			}
 
 		}
+		else {
+			switch (info->field_id) {
+			case E_CONTACT_CATEGORIES: {
+				EVCardAttribute *attr = e_contact_get_first_attr (contact, "CATEGORIES");
+				char **split, **s;
+				const char *str;
+
+				if (attr)
+					e_vcard_attribute_remove_values (attr);
+				else
+					/* we didn't find it - add a new attribute */
+					attr = e_vcard_attribute_new (NULL, "CATEGORIES");
+
+				str = g_value_get_string (value);
+				if (str) {
+					split = g_strsplit (str, ",", 0);
+					if (split) {
+						for (s = split; *s; s++) {
+							e_vcard_attribute_add_value (attr, g_strstrip (*s));
+						}
+						g_strfreev (split);
+					} else
+						e_vcard_attribute_add_value (attr, str);
+				}
+				break;
+			}
+			default:
+				g_warning ("unhandled synthetic field 0x%02x", info->field_id);
+				break;
+			}
+		}
 	}
 	else if (info->t & E_CONTACT_FIELD_TYPE_STRUCT || info->t & E_CONTACT_FIELD_TYPE_GETSET) {
 		EVCardAttribute *attr = e_contact_get_first_attr (contact, info->vcard_field_name);
-		void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : g_value_get_pointer (value);
+		void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : (char*)g_value_get_string (value);
 
-		if (attr) {
+		if (attr && data) {
 			printf ("overwriting existing %s\n", info->vcard_field_name);
 			/* remove all existing values and parameters.
 			   the setter will add the correct ones */
 			e_vcard_attribute_remove_values (attr);
 			e_vcard_attribute_remove_params (attr);
+
+			info->struct_setter (contact, attr, data);
 		}
-		else {
+		else if (data) {
 			printf ("adding new %s\n", info->vcard_field_name);
 			attr = e_vcard_attribute_new (NULL, info->vcard_field_name);
 
 			e_vcard_add_attribute (E_VCARD (contact), attr);
-		}
 
-		info->struct_setter (contact, attr, data);
+			info->struct_setter (contact, attr, data);
+		}
+		else if (attr) {
+			printf ("removing %s\n", info->vcard_field_name);
+
+			e_vcard_remove_attribute (E_VCARD (contact), attr);
+		}
 	}
 	else if (info->t & E_CONTACT_FIELD_TYPE_BOOLEAN) {
 		EVCardAttribute *attr;
@@ -932,15 +1006,8 @@ e_contact_find_attribute_with_types (EContact *contact, const char *attr_name, c
 		EVCardAttribute *attr = l->data;
 		const char *name, *group;
 
-		if (!type_needed1)
-			found_needed1 = TRUE;
-		else
-			found_needed1 = FALSE;
-
-		if (!type_needed2)
-			found_needed2 = TRUE;
-		else
-			found_needed2 = FALSE;
+		found_needed1 = (type_needed1 == NULL);
+		found_needed2 = (type_needed2 == NULL);
 
 		group = e_vcard_attribute_get_group (attr);
 		name = e_vcard_attribute_get_name (attr);
@@ -956,10 +1023,26 @@ e_contact_find_attribute_with_types (EContact *contact, const char *attr_name, c
 				if (!strcasecmp (name, EVC_TYPE)) {
 					GList *values = e_vcard_attribute_param_get_values (param);
 					if (values && values->data) {
-						if (!found_needed1 && !strcasecmp ((char*)values->data, type_needed1))
+						gboolean matches = FALSE;
+
+						if (!found_needed1 && !strcasecmp ((char*)values->data, type_needed1)) {
 							found_needed1 = TRUE;
-						else if (!found_needed2 && !strcasecmp ((char*)values->data, type_needed2))
+							matches = TRUE;
+						}
+						else if (!found_needed2 && !strcasecmp ((char*)values->data, type_needed2)) {
 							found_needed2 = TRUE;
+							matches = TRUE;
+						}
+
+						if (!matches) {
+							/* this is to enforce that we find an attribute
+							   with *only* the TYPE='s we need.  This may seem like
+							   an odd restriction but it's the only way at present to
+							   implement the Other Fax and Other Phone attributes. */
+							found_needed1 =
+								found_needed2 = FALSE;
+							break;
+						}
 					}
 				}
 
@@ -1111,7 +1194,11 @@ e_contact_get_property (GObject *object,
 		if (attr)
 			rv = info->struct_getter (contact, attr);
 
-		g_value_set_pointer (value, rv);
+		if (info->t & E_CONTACT_FIELD_TYPE_STRUCT)
+			g_value_set_boxed (value, rv);
+		else
+			g_value_set_string (value, (char*)rv);
+			
 	}
 
 	else if (info->t & E_CONTACT_FIELD_TYPE_SYNTHETIC) {
@@ -1136,8 +1223,30 @@ e_contact_get_property (GObject *object,
 			g_value_set_string (value, str);
 			break;
 		}
+		case E_CONTACT_CATEGORIES: {
+			EVCardAttribute *attr = e_contact_get_first_attr (contact, "CATEGORIES");
+			char *rv = NULL;
+
+			if (attr) {
+				GString *str = g_string_new ("");
+				GList *v = e_vcard_attribute_get_values (attr);
+				while (v) {
+					g_string_append (str, (char*)v->data);
+					v = v->next;
+					if (v)
+						g_string_append_c (str, ',');
+				}
+
+				rv = g_string_free (str, FALSE);
+			}
+
+			g_value_set_string (value, rv);
+			g_free (rv);
+			break;
+		}
 		default:
 			g_warning ("unhandled synthetic field 0x%02x", info->field_id);
+			break;
 		}
 	}
 	else {
@@ -1182,8 +1291,11 @@ e_contact_new (void)
 EContact*
 e_contact_new_from_vcard  (const char *vcard)
 {
-	EContact *contact = g_object_new (E_TYPE_CONTACT, NULL);
+	EContact *contact;
 
+	g_return_val_if_fail (vcard != NULL, NULL);
+
+	contact = g_object_new (E_TYPE_CONTACT, NULL);
 	e_vcard_construct (E_VCARD (contact), vcard);
 
 	return contact;
@@ -1192,9 +1304,13 @@ e_contact_new_from_vcard  (const char *vcard)
 EContact*
 e_contact_duplicate (EContact *contact)
 {
-	char *vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
-	EContact *c = e_contact_new_from_vcard (vcard);
+	char *vcard;
+	EContact *c;
 
+	g_return_val_if_fail (E_IS_CONTACT (contact), NULL);
+
+	vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
+	c = e_contact_new_from_vcard (vcard);
 	g_free (vcard);
 
 	return c;
@@ -1328,6 +1444,8 @@ e_contact_name_from_string (const char *name_str)
 	EContactName *name = e_contact_name_new();
 	ENameWestern *western;
 
+	g_return_val_if_fail (name_str != NULL, NULL);
+
 	western = e_name_western_parse (name_str ? name_str : "");
 	
 	name->prefixes   = g_strdup (western->prefix);
@@ -1344,7 +1462,11 @@ e_contact_name_from_string (const char *name_str)
 EContactName*
 e_contact_name_copy (EContactName *n)
 {
-	EContactName *name = e_contact_name_new();
+	EContactName *name;
+
+	g_return_val_if_fail (n != NULL, NULL);
+
+	name = e_contact_name_new();
 	
 	name->prefixes   = g_strdup (n->prefixes);
 	name->given      = g_strdup (n->given);
@@ -1385,9 +1507,12 @@ e_contact_name_get_type (void)
 EContactDate*
 e_contact_date_from_string (const char *str)
 {
-	EContactDate* date = e_contact_date_new();
+	EContactDate* date;
 	int length;
 
+	g_return_val_if_fail (str != NULL, NULL);
+
+	date = e_contact_date_new();
 	length = strlen(str);
 	
 	if (length == 10 ) {
