@@ -129,9 +129,10 @@ CORBA_sequence_ ## type ## _realloc (CORBA_sequence_ ## type *seq,	\
 				     CORBA_unsigned_long      new_max)	\
 {									\
 	type *new_buf;							\
-									\
+	int i;								\
 	new_buf = CORBA_sequence_ ## type ## _allocbuf (new_max);	\
-	memcpy (new_buf, seq->_buffer, seq->_maximum * sizeof (type));	\
+	for (i = 0; i < seq->_maximum; i ++)				\
+		new_buf[i] = CORBA_string_dup (seq->_buffer[i]);	\
 	CORBA_free (seq->_buffer);					\
 	seq->_buffer = new_buf;						\
 	seq->_maximum = new_max;					\
@@ -394,7 +395,9 @@ e_data_book_view_new (EBookBackend *backend,
 {
 	EDataBookView *book_view;
 
-	book_view = g_object_new (E_TYPE_DATA_BOOK_VIEW, NULL);
+	book_view = g_object_new (E_TYPE_DATA_BOOK_VIEW,
+				  "poa", bonobo_poa_get_threaded (ORBIT_THREAD_HINT_PER_REQUEST, NULL),
+				  NULL);
 	
 	e_data_book_view_construct (book_view, backend, listener, card_query, card_sexp);
 
