@@ -28,12 +28,14 @@
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <bonobo/bonobo-i18n.h>
 #include <libedataserver/e-xml-hash-utils.h>
 #include <libedata-cal/e-cal-backend-cache.h>
 #include <libedata-cal/e-cal-backend-util.h>
 #include <libecal/e-cal-component.h>
 #include <libecal/e-cal-time-util.h>
 #include "e-cal-backend-groupwise.h"
+#include "e-cal-backend-groupwise-utils.h"
 #include "e-gw-connection.h"
 
 /* Private part of the CalBackendGroupwise structure */
@@ -62,32 +64,42 @@ static ECalBackendClass *parent_class = NULL;
 
 /* Initialy populate the cache from the server */
 static EGwConnectionStatus
-populate_cache (ECalBackendGroupwisePrivate *priv)
+populate_cache (ECalBackendGroupwise *cbgw)
 {
+	ECalBackendGroupwisePrivate *priv;
 	EGwConnectionStatus status;
         ECalComponent *comp;
 	const char *uid;
 	char *rid;
         GList *list = NULL, *l;
 
+	priv = cbgw->priv;
+
         /* get all the objects from the server */
         status = e_gw_connection_get_items (priv->cnc, priv->container_id, NULL, &list);
         if (status != E_GW_CONNECTION_STATUS_OK) {
                 g_list_free (list);
                 return status;
         }
-        
+
         for (l = list; l != NULL; l = g_list_next(l)) {
-                comp = E_CAL_COMPONENT (l->data);
-		e_cal_component_get_uid (comp, &uid);
-                rid = g_strdup (e_cal_component_get_recurid_as_string (comp));
-                e_cal_component_commit_sequence (comp);
-		e_cal_backend_cache_put_component (priv->cache, comp);
-                g_free (rid);
-                g_free (comp);
+		EGwItem *item;
+
+		item = E_GW_ITEM (l->data);
+		comp = e_gw_item_to_cal_component (item);
+		g_object_unref (item);
+		if (E_IS_CAL_COMPONENT (comp)) {
+			e_cal_component_commit_sequence (comp);
+			e_cal_component_get_uid (comp, &uid);
+			rid = g_strdup (e_cal_component_get_recurid_as_string (comp));
+			e_cal_backend_cache_put_component (priv->cache, comp);
+			g_free (rid);
+			g_object_unref (comp);
+		}
         }
         
         g_list_free (list);
+
         return E_GW_CONNECTION_STATUS_OK;        
 }
 
@@ -108,6 +120,60 @@ convert_uri (const char *gw_uri)
 	return vuri;
 }
 
+static EGwConnectionStatus
+get_deltas (gpointer handle)
+{
+ 	ECalBackendGroupwise *cbgw;
+	EGwConnection *cnc; 
+ 	ECalBackendCache *cache; 
+        EGwConnectionStatus status; 
+	GSList *deletes = NULL, *updates = NULL, *adds = NULL, *l;
+        
+	cbgw = (ECalBackendGroupwise *)handle;
+ 	cnc = cbgw->priv->cnc; 
+ 	cache = cbgw->priv->cache; 
+
+	/* Call e-gw-connection_get_deltas*/
+	status = e_gw_connection_get_deltas (cnc, &adds, &deletes, &updates);
+
+	if (status != E_GW_CONNECTION_STATUS_OK) {
+		g_warning (G_STRLOC ": Could not refresh the cache");
+		return status;
+	}
+	
+	if (deletes) {
+		for (l = deletes; l != NULL; l = g_slist_next(l)) {
+			if (!e_cal_backend_cache_remove_component (cache, (char *)l->data, NULL)) 
+                                         g_message ("Could not remove %s", (char *)l->data);
+		}
+	}
+
+	if (adds) {
+		for (l = adds; l != NULL; l = g_slist_next (l)) {
+			EGwItem *item = (EGwItem *) l->data;
+			ECalComponent *comp = e_gw_item_to_cal_component (item);
+			if (!comp)
+				g_message ("Invalid component returned");
+			else if (!e_cal_backend_cache_put_component (cache, comp)) 
+				g_message ("Could not add the component");
+		}
+	}	
+	if (updates) {
+		for (l = updates; l != NULL; l = g_slist_next (l)) {
+			EGwItem *item = (EGwItem *) l->data;
+			ECalComponent *comp = e_cal_backend_cache_get_component (cache, e_gw_item_get_id (item), NULL);
+			if (!comp) /* FIXME  Error in updates. Skipping the element*/
+				continue;
+			/* FIXME  currently, just overwrite the fields with the
+			 * update.*/
+			e_cal_backend_cache_remove_component (cache, e_gw_item_get_id (item), NULL);
+			e_cal_backend_cache_put_component (cache, e_gw_item_to_cal_component (item));
+				
+		}
+	}
+        return E_GW_CONNECTION_STATUS_OK;        
+}
+
 static ECalBackendSyncStatus
 connect_to_server (ECalBackendGroupwise *cbgw)
 {
@@ -147,23 +213,27 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 				g_free (priv->container_id);
 
 			kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw));
-			if (kind == ICAL_VEVENT_COMPONENT)
+			if (kind == ICAL_VEVENT_COMPONENT) {
 				priv->container_id = e_gw_connection_get_container_id (priv->cnc, "Calendar");
-			else if (kind == ICAL_VTODO_COMPONENT)
+				e_source_set_name (e_cal_backend_get_source (E_CAL_BACKEND (cbgw)), _("Calendar"));
+			} else if (kind == ICAL_VTODO_COMPONENT) {
 				priv->container_id = e_gw_connection_get_container_id (priv->cnc, "Checklist");
-			else
+				e_source_set_name (e_cal_backend_get_source (E_CAL_BACKEND (cbgw)), _("Checklist"));
+			} else
 				priv->container_id = NULL;
 
 			/* Populate the cache for the first time.*/
 			/* start a timed polling thread set to 10 minutes*/
-			status = populate_cache (priv);
+			status = populate_cache (cbgw);
 			if (status != E_GW_CONNECTION_STATUS_OK) {
 				g_object_unref (priv->cnc);
 				priv->cnc = NULL;
 				g_warning (G_STRLOC ": Could not populate the cache");
 				return GNOME_Evolution_Calendar_PermissionDenied;
 			} else {
-				g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) e_gw_connection_get_deltas, (gpointer) cbgw);
+				g_object_ref (priv->cnc);
+				g_object_ref (priv->cache);
+				g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas, (gpointer) cbgw);
 				priv->mode = CAL_MODE_REMOTE;
 			}
 
@@ -481,10 +551,15 @@ e_cal_backend_groupwise_get_object (ECalBackendSync *backend, EDataCal *cal, con
 	comp = e_cal_backend_cache_get_component (priv->cache, uid, rid);
 	if (comp) {
 		g_mutex_unlock (priv->mutex);
-		*object = e_cal_component_get_as_string (comp);
+		if (e_cal_backend_get_kind (E_CAL_BACKEND (backend)) ==
+		    icalcomponent_isa (e_cal_component_get_icalcomponent (comp)))
+			*object = e_cal_component_get_as_string (comp);
+		else
+			*object = NULL;
+
 		g_object_unref (comp);
 
-		return GNOME_Evolution_Calendar_Success;
+		return *object ? GNOME_Evolution_Calendar_Success : GNOME_Evolution_Calendar_ObjectNotFound;
 	}
 
 	g_mutex_unlock (priv->mutex);
@@ -568,9 +643,12 @@ e_cal_backend_groupwise_get_object_list (ECalBackendSync *backend, EDataCal *cal
         for (l = components; l != NULL; l = l->next) {
                 ECalComponent *comp = E_CAL_COMPONENT (l->data);
 
-		if ((!search_needed) ||
-		    (e_cal_backend_sexp_match_comp (cbsexp, comp, backend))) {
-			*objects = g_list_append (*objects, e_cal_component_get_as_string (comp));
+		if (e_cal_backend_get_kind (E_CAL_BACKEND (backend)) ==
+		    icalcomponent_isa (e_cal_component_get_icalcomponent (comp))) {
+			if ((!search_needed) ||
+			    (e_cal_backend_sexp_match_comp (cbsexp, comp, E_CAL_BACKEND (backend)))) {
+				*objects = g_list_append (*objects, e_cal_component_get_as_string (comp));
+			}
 		}
         }
 
@@ -784,6 +862,11 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
 
+	if (e_cal_backend_get_kind (E_CAL_BACKEND (backend)) != icalcomponent_isa (icalcomp)) {
+		icalcomponent_free (icalcomp);
+		return GNOME_Evolution_Calendar_InvalidObject;
+	}
+
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomp);
 
@@ -871,8 +954,8 @@ e_cal_backend_groupwise_remove_object (ECalBackendSync *backend, EDataCal *cal,
 	if (priv->mode == CAL_MODE_REMOTE) {
 		EGwConnectionStatus status;
 
-		/* FIXME: deal with recurrences, and pass the correct 'container' */
-		status = e_gw_connection_remove_item (priv->cnc, NULL, uid);
+		/* FIXME: deal with recurrences */
+		status = e_gw_connection_remove_item (priv->cnc, priv->container_id, uid);
 		if (status != E_GW_CONNECTION_STATUS_OK)
 			return GNOME_Evolution_Calendar_OtherError;
 	}
