@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* Evolution calendar - iCalendar http backend
  *
  * Copyright (C) 2003 Novell, Inc.
@@ -58,13 +59,17 @@ struct _ECalBackendHttpPrivate {
 
 	/* Soup handles for remote file */
 	SoupSession *soup_session;
-	SoupMessage *soup_message;
+
+	/* Reload */
+	guint reload_timeout_id;
+	guint is_loading : 1;
 };
 
 
 
 static void e_cal_backend_http_dispose (GObject *object);
 static void e_cal_backend_http_finalize (GObject *object);
+static gboolean begin_retrieval_cb (ECalBackendHttp *cbhttp);
 
 static ECalBackendSyncClass *parent_class;
 
@@ -99,12 +104,6 @@ e_cal_backend_http_finalize (GObject *object)
 
 	/* Clean up */
 
-	if (priv->soup_message) {
-		soup_session_cancel_message (priv->soup_session, priv->soup_message);
-		g_object_unref (priv->soup_message);
-		priv->soup_message = NULL;
-	}
-
 	if (priv->cache) {
 		g_object_unref (priv->cache);
 		priv->cache = NULL;
@@ -116,6 +115,7 @@ e_cal_backend_http_finalize (GObject *object)
 	}
 
 	if (priv->soup_session) {
+		soup_session_abort (priv->soup_session);
 		g_object_unref (priv->soup_session);
 		priv->soup_session = NULL;
 	}
@@ -183,7 +183,7 @@ static gchar *
 webcal_to_http_method (const gchar *webcal_str)
 {
 	if (strncmp ("webcal://", webcal_str, sizeof ("webcal://") - 1))
-		return NULL;
+		return g_strdup (webcal_str);
 
 	return g_strconcat ("http://", webcal_str + sizeof ("webcal://") - 1, NULL);
 }
@@ -193,37 +193,55 @@ retrieval_done (SoupMessage *msg, ECalBackendHttp *cbhttp)
 {
 	ECalBackendHttpPrivate *priv;
 	icalcomponent *icalcomp, *subcomp;
+	const char *newuri;
 	char *str;
 
 	priv = cbhttp->priv;
 
+	priv->is_loading = FALSE;
+	g_message ("Retrieval done.\n");
+
+	/* Handle redirection ourselves */
+	if (SOUP_STATUS_IS_REDIRECTION (msg->status_code)) {
+		newuri = soup_message_get_header (msg->response_headers,
+						  "Location");
+
+		g_message ("Redirected to %s\n", newuri);
+
+		if (newuri) {
+			g_free (priv->uri);
+
+			priv->uri = webcal_to_http_method (newuri);
+			begin_retrieval_cb (cbhttp);
+		} else {
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp),
+						    _("Redirected to Invalid URI"));
+		}
+
+		return;
+	}
+
 	/* check status code */
-	if (priv->soup_message->status_code != SOUP_STATUS_OK) {
+	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
 		e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp),
-					    soup_status_get_phrase (priv->soup_message->status_code));
-		g_object_unref (priv->soup_message);
-		priv->soup_message = NULL;
+					    soup_status_get_phrase (msg->status_code));
 		return;
 	}
 
 	/* get the calendar from the response */
-	str = g_malloc0 (priv->soup_message->response.length + 1);
-	strncpy (str, msg->response.body, priv->soup_message->response.length);
+	str = g_malloc0 (msg->response.length + 1);
+	strncpy (str, msg->response.body, msg->response.length);
 	icalcomp = icalparser_parse_string (str);
 	g_free (str);
 
 	if (!icalcomp) {
 		e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp), _("Bad file format."));
-		g_object_unref (priv->soup_message);
-		priv->soup_message = NULL;
 		return;
 	}
 
 	if (icalcomponent_isa (icalcomp) != ICAL_VCALENDAR_COMPONENT) {
 		e_cal_backend_notify_error (E_CAL_BACKEND (cbhttp), _("Not a calendar."));
 		icalcomponent_free (icalcomp);
-		g_object_unref (priv->soup_message);
-		priv->soup_message = NULL;
 		return;
 	}
 
@@ -246,42 +264,95 @@ retrieval_done (SoupMessage *msg, ECalBackendHttp *cbhttp)
 
 	/* free memory */
 	icalcomponent_free (icalcomp);
-	g_object_unref (priv->soup_message);
-	priv->soup_message = NULL;
+
+	g_message ("Retrieval really done.\n");
 }
 
+static gboolean reload_cb                  (ECalBackendHttp *cbhttp);
+static void     maybe_start_reload_timeout (ECalBackendHttp *cbhttp);
+
 static gboolean
 begin_retrieval_cb (ECalBackendHttp *cbhttp)
 {
-	gchar *source_uri_str;
 	ECalBackendHttpPrivate *priv;
+	SoupMessage *soup_message;
 
 	priv = cbhttp->priv;
 
 	if (priv->mode != CAL_MODE_REMOTE)
 		return TRUE;
 
-	if (priv->soup_message != NULL)
+	maybe_start_reload_timeout (cbhttp);
+
+	g_message ("Starting retrieval...\n");
+
+	if (priv->is_loading)
 		return FALSE;
 
+	priv->is_loading = TRUE;
+
 	/* create the Soup session if not already created */
 	if (!priv->soup_session) {
 		priv->soup_session = soup_session_async_new ();
 	}
 
-	source_uri_str = webcal_to_http_method (e_cal_backend_get_uri (E_CAL_BACKEND (cbhttp)));
-	if (!source_uri_str)
-		source_uri_str = g_strdup (e_cal_backend_get_uri (E_CAL_BACKEND (cbhttp)));
+	if (priv->uri == NULL)
+		priv->uri = webcal_to_http_method (e_cal_backend_get_uri (E_CAL_BACKEND (cbhttp)));
 
 	/* create message to be sent to server */
-	priv->soup_message = soup_message_new (SOUP_METHOD_GET, source_uri_str);
-	g_free (source_uri_str);
-	soup_session_queue_message (priv->soup_session, priv->soup_message,
+	soup_message = soup_message_new (SOUP_METHOD_GET, priv->uri);
+	soup_message_set_flags (soup_message, SOUP_MESSAGE_NO_REDIRECT);
+
+	soup_session_queue_message (priv->soup_session, soup_message,
 				    (SoupMessageCallbackFn) retrieval_done, cbhttp);
 
+	g_message ("Retrieval started.\n");
 	return FALSE;
 }
 
+static gboolean
+reload_cb (ECalBackendHttp *cbhttp)
+{
+	ECalBackendHttpPrivate *priv;
+
+	priv = cbhttp->priv;
+
+	if (priv->is_loading)
+		return TRUE;
+
+	g_message ("Reload!\n");
+
+	priv->reload_timeout_id = 0;
+	begin_retrieval_cb (cbhttp);
+	return FALSE;
+}
+
+static void
+maybe_start_reload_timeout (ECalBackendHttp *cbhttp)
+{
+	ECalBackendHttpPrivate *priv;
+	ESource *source;
+	const gchar *refresh_str;
+
+	priv = cbhttp->priv;
+
+	g_message ("Setting reload timeout.\n");
+
+	if (priv->reload_timeout_id)
+		return;
+
+	source = e_cal_backend_get_source (E_CAL_BACKEND (cbhttp));
+	if (!source) {
+		g_warning ("Could not get source for ECalBackendHttp reload.");
+		return;
+	}
+
+	refresh_str = e_source_get_property (source, "refresh");
+
+	priv->reload_timeout_id = g_timeout_add ((refresh_str ? atoi (refresh_str) : 30) * 60000,
+						 (GSourceFunc) reload_cb, cbhttp);
+}
+
 /* Open handler for the file backend */
 static ECalBackendSyncStatus
 e_cal_backend_http_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
@@ -729,6 +800,7 @@ e_cal_backend_http_init (ECalBackendHttp *cbhttp, ECalBackendHttpClass *class)
 	cbhttp->priv = priv;
 
 	priv->uri = NULL;
+	priv->reload_timeout_id = 0;
 }
 
 /* Class initialization function for the file backend */
