@@ -22,6 +22,7 @@
 #define E_CAL_H
 
 #include <glib-object.h>
+#include <libedataserver/e-source-list.h>
 #include <libedataserver/e-source.h>
 #include <libecal/e-cal-recur.h>
 #include <libecal/e-cal-util.h>
@@ -82,10 +83,9 @@ struct _ECalClass {
 	void (* cal_opened) (ECal *ecal, ECalendarStatus status);
 	void (* cal_set_mode) (ECal *ecal, ECalSetModeStatus status, CalMode mode);	
 
-	void (* backend_error) (ECal *ecal, const char *message);
-
 	void (* categories_changed) (ECal *ecal, GPtrArray *categories);
 
+	void (* backend_error) (ECal *ecal, const char *message);
 	void (* backend_died) (ECal *ecal);
 };
 
@@ -102,12 +102,14 @@ GType cal_mode_enum_get_type (void);
 
 ECal *e_cal_new (ESource *source, ECalSourceType type);
 ECal *e_cal_new_from_uri (const gchar *uri, ECalSourceType type);
+ECal *e_cal_new_system_calendar (void);
+ECal *e_cal_new_system_tasks (void);
 
 void e_cal_set_auth_func (ECal *ecal, ECalAuthFunc func, gpointer data);
 
 gboolean e_cal_open (ECal *ecal, gboolean only_if_exists, GError **error);
 void e_cal_open_async (ECal *ecal, gboolean only_if_exists);
-gboolean e_cal_remove_calendar (ECal *ecal, GError **error);
+gboolean e_cal_remove (ECal *ecal, GError **error);
 
 GList *e_cal_uri_list (ECal *ecal, CalMode mode);
 
@@ -187,6 +189,12 @@ char* e_cal_get_component_as_string (ECal *ecal, icalcomponent *icalcomp);
 
 const char * e_cal_get_error_message (ECalendarStatus status);
 
+/* Calendar/Tasks Discovery */
+gboolean    e_cal_open_default (ECal **ecal, ECalSourceType type, ECalAuthFunc func, gpointer data, GError **error);
+gboolean    e_cal_set_default (ECal  *ecal, GError **error);
+gboolean    e_cal_set_default_source (ESource *source, ECalSourceType type, GError **error);
+gboolean    e_cal_get_sources (ESourceList **sources, ECalSourceType type, GError **error);
+
 G_END_DECLS
 
 #endif
